import logging
from brian_dashboard_manager.grafana.utils.request import \
    AdminRequest, \
    TokenRequest
from brian_dashboard_manager.grafana.organization import \
    get_organizations, \
    create_organization, \
    create_api_token, \
    delete_api_token, \
    delete_expired_api_tokens
from brian_dashboard_manager.grafana.dashboard import \
    get_dashboard_definitions, \
    create_dashboard
from brian_dashboard_manager.grafana.datasource import \
    get_missing_datasource_definitions,\
    create_datasource


logger = logging.getLogger(__name__)


def provision(config):

    request = AdminRequest(**config)
    all_orgs = get_organizations(request)

    organizations_to_provision = config.get('organizations', [])

    missing = (name for name in organizations_to_provision
               if name not in [org['name'] for org in all_orgs])

    for org in missing:
        org_data = create_organization(request, org)
        all_orgs.append(org_data)

    for org in all_orgs:
        org_id = org['id']
        delete_expired_api_tokens(request, org_id)
        token = create_api_token(request, org_id)
        token_request = TokenRequest(token=token['key'], **config)

        logger.info(f'--- Provisioning org {org["name"]} (ID #{org_id}) ---')

        # Provision missing data sources
        for datasource in get_missing_datasource_definitions(token_request):
            if datasource:
                ds = create_datasource(token_request,
                                       datasource,
                                       config.get('datasources'))
                if ds:
                    logger.info(
                        f'Provisioned datasource: {datasource["name"]}')

        # Provision dashboards, overwriting existing ones.
        for dashboard in get_dashboard_definitions():
            create_dashboard(token_request, dashboard)

        delete_api_token(request, org_id, token['id'])

    return all_orgs
